/***************************************************************************
 *   Copyright (C) 2005 by Peter Müller   *
 *   pmueller@cs.tu-berlin.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef COMET_H
#define COMET_H

#include "cometenscene.h"
#include "particlesystem.h"
#include "lightning.h"
#include "decal.h"
#include "explosion.h"

class CometenScene;

// 1. Comet->process
// 2. Collision
// 3. Move linear
   
class Comet {
public:      		
	Comet(CometenScene * _scene,
		const Vec3 & _p = Vec3::ZERO,
		const Vec3 & _v = Vec3::ZERO,
		float _r = 1.0f):
	scene(_scene),
	pos(_p),
	vel(_v),
	radius(_r) {};
		
	virtual ~Comet() {};
	
	virtual void process(float time, float deltat) = 0;
	virtual void render() = 0;
	// new pos is calculated in Cometenscene::cometHit*()
	virtual void hitPlane(float time, const Plane & plane, const Vec3 & hitpoint) = 0;
	virtual void destroy(float time) = 0;
	virtual void kill() = 0;
	// destroyed => no moving, no collision, done => delete
	virtual bool destroyed() = 0;
	virtual bool done() = 0;
	
	float getRadius() { return radius; }
	void setRadius(float r) { radius = r; }
	
	Vec3 getPos() { return pos; }
	void setPos(const Vec3& p) { pos = p; }
	
	Vec3 getVel() { return vel; }
	void setVel(const Vec3& v) { vel = v; }	

protected:
	CometenScene * scene;
	Vec3 pos;
	Vec3 vel;
	float radius;
};	



class PComet : public Comet {
public:
	PComet(CometenScene * _scene, float _time, float _life_time, int num_children,
		   const Vec3 & _p, const Vec3 & _v, const Vec3 & _a, float size, 
		   const Vec4 & _acolor, const Vec4 & _ecolor);
	virtual ~PComet();
	
	virtual void process(float time, float deltat);
	virtual void render();
	virtual void hitPlane(float time, const Plane & plane, const Vec3 & hitpoint);
	virtual void destroy(float time);
	virtual void kill();
	virtual bool destroyed();
	virtual bool done();
	
private:
	bool _destroyed;
	float start_time;
	float life_time;
	ParticleSystem *psystem;
	Decal *decal;
	float size;
	int num_children;
	Vec3 accel;
	Vec4 acolor, ecolor;
};


#endif
