/***************************************************************************
 *   Copyright (C) 2005 by Peter Müller   *
 *   pmueller@cs.tu-berlin.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DECAL_H
#define DECAL_H

#define GL_GLEXT_PROTOTYPES
#include <GL/gl.h>
#include <GL/glu.h>

#include "effect.h"

/**
@author Peter Müller
*/
class Decal : public Effect
{
public:	
	// normal must be normalized
	Decal(GLuint texture, const Vec3 & pos, bool billboard, 
	      const Vec3 & normal, const Vec2 & size, const Vec4 & color,
	      float time, float fadein_time, float fadeout_time);
	
    virtual ~Decal();

	virtual void process(float time);
	virtual void render(const Vec3 & campos);
	virtual void stop();
	virtual bool done();
	
protected:
	bool billboard;
	const Vec3 normal;
	Vec2 size;
	Vec4 color;
	GLuint texture;
	float transparency;
	float start_time;
	bool stopped;
	float end_time;
	float fadein_time;
	float fadeout_time;
	float last_process_time;
};

class TimeLimitDecal : public Decal {
public:
	// normal must be normalized
	TimeLimitDecal(GLuint texture, const Vec3 & pos, bool billboard, const Vec3 & normal, 
	      		const Vec2 & start_size, const Vec2 & end_size, const Vec4 & color,
	            float time, float life_time, float fadein_time, float fadeout_time);
	
	virtual ~TimeLimitDecal();
	
	virtual void process(float time, float deltat);
	
protected:
	Vec2 start_size, end_size;
};	

#endif
