/***************************************************************************
 *   Copyright (C) 2005 by Peter Müller   *
 *   pmueller@cs.tu-berlin.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kglcometen3.h"
#include "cometen_math.h"


KGLCometen3::KGLCometen3(QWidget *parent, const char *name)
: QGLWidget(parent, name)
{
	scene = new CometenScene();
}

KGLCometen3::~KGLCometen3() {
	delete scene;
}

void KGLCometen3::start(KConfig* config) 
{
	config->setGroup( "Settings" );
	
	do_blitz = config->readBoolEntry( "lightning", true );
	do_matrix = config->readBoolEntry( "matrix", true );
	do_slowmotion = config->readBoolEntry( "slowmotion", true );
	interval = clamp(0.1, 10.0, config->readDoubleNumEntry("interval", 1.25));
	colors = config->readNumEntry("color", 3);
	bgtype = config->readNumEntry("bgtype", 0);
	bgfile = config->readPathEntry("bgfile","");
	bgsize = config->readNumEntry("bgsize", 1);
	maxfps = clamp(30, 999, config->readNumEntry("maxfps", 60));
	
	startTimer(1000/maxfps);
}

void KGLCometen3::initializeGL() {
	GLuint bg;
	QImage image;
	
	if (bgtype == 1) {
		if (!image.load(bgfile)) {
			image = QPixmap::grabWindow(QApplication::desktop()->winId());
		}
	}
	else {
		image = QPixmap::grabWindow(QApplication::desktop()->winId());
	}
	
	switch (bgsize) {
	case 0:
		image = image.smoothScale(128, 128, QImage::ScaleFree);
		break;
	case 1:
		image = image.smoothScale(256, 256, QImage::ScaleFree);
		break;
	case 2:
		image = image.smoothScale(512, 512, QImage::ScaleFree);
		break;
	case 3:
	default:
		break;
	}
		
	image = QGLWidget::convertToGLFormat(image);
	
	glEnable(GL_TEXTURE_2D);
	glGenTextures(1, &bg);
	glBindTexture(GL_TEXTURE_2D, bg); 
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	
	glTexImage2D(GL_TEXTURE_2D, 0, 3, image.width(), image.height(),
	             0, GL_RGBA, GL_UNSIGNED_BYTE, image.bits());	
	
	t.start();
	scene->init(t.elapsed() / 1000.0f, bg, colors, interval, 1.0f, 
	            do_matrix, do_slowmotion, do_blitz);
}

void KGLCometen3::resizeGL(int res_x, int res_y) {
	scene->resize(res_x, res_y);
}

void KGLCometen3::paintGL() {
	scene->render();
}

void KGLCometen3::timerEvent(QTimerEvent * event) {
	scene->process(t.elapsed() / 1000.0f);
	updateGL();
}

GLuint load_texture(const string& file) {
	QImage image;
	GLuint tex = 0;

	if (image.load(KGlobal::dirs()->findResource("data", 
	      QString("kcometen3/pixmaps/") + file))) {
		image = QGLWidget::convertToGLFormat(image);
		
		glEnable(GL_TEXTURE_2D);
		glGenTextures(1, &tex);
		glBindTexture(GL_TEXTURE_2D, tex); 
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, image.width(), image.height(),
		             0, GL_RGBA, GL_UNSIGNED_BYTE, image.bits());
	}
	
	return tex;
}
		
		

#include "kglcometen3.moc"
